Package: hfhub
Title: Hugging Face Hub Interface
Version: 0.1.2
Authors@R: c(
    person("Tomasz", "Kalinowski", , "tomasz@posit.co", role = c("ctb", "cre")),
    person("Daniel", "Falbel", , "daniel@posit.co", role = c("aut")),
    person("Christophe", "Regouby", , "christophe.regouby@free.fr", role = c("ctb")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
    )
Description: Provides functionality to download and cache files from 'Hugging Face Hub' <https://huggingface.co/models>. 
    Uses the same caching structure so files can be shared between different client libraries.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: httr, filelock, fs, cli, withr, curl, glue, urltools, rlang
Suggests: testthat (>= 3.0.0), jsonlite
Config/testthat/edition: 3
URL: https://mlverse.github.io/hfhub/
BugReports: https://github.com/mlverse/hfhub/issues
NeedsCompilation: no
Packaged: 2026-04-15 14:23:17 UTC; dfalbel
Author: Tomasz Kalinowski [ctb, cre],
  Daniel Falbel [aut],
  Christophe Regouby [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-15 14:50:02 UTC
Built: R 4.4.3; ; 2026-04-17 07:35:56 UTC; unix
