# snapshot

    Code
      p <- hub_snapshot("dfalbel/cran-packages", repo_type = "dataset",
        allow_patterns = "\\.R")
    Message
      i Snapshotting files 0/4
      v Snapshotting files 4/4 [0ms]
      

